// DemoApplication.h

#ifndef DEMO_APPLICATION_H
#define DEMO_APPLICATION_H

#include "ExampleApplication.h"
#include "DemoFrameListener.h"

#define CAMERA_POSITION Vector3(0, 10, 30)
#define CAMERA_LOOK_AT Vector3(0, 0, 0)
#define CAMERA_NEAR_CLIP_DISTANCE 5
#define CAMERA_NODE_POSITION Vector3(2000, 100, 2000)

#define AMBIENT_LIGHT ColourValue(0.5, 0.5, 0.5)
#define LIGHT_DIRECTION Vector3(0.2, -1, 0.2)

class DemoApplication : public ExampleApplication
{
protected:
	SceneNode* mCameraNode;

public:
	DemoApplication()
	{
	}

	virtual ~DemoApplication()
	{
	}

protected:

	virtual void createFrameListener(void)
    {
        mFrameListener= new DemoFrameListener(mSceneMgr, mWindow, mCamera);
        mFrameListener->showDebugOverlay(true);
        mRoot->addFrameListener(mFrameListener);
    }

	virtual void chooseSceneManager()
	{
		mSceneMgr = mRoot->createSceneManager(ST_EXTERIOR_CLOSE, "SceneManagerInstance");
	}

	virtual void createCamera()
	{
		mCamera = mSceneMgr->createCamera("Camera");

		mCameraNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("CameraNode");
		mCameraNode->attachObject(mCamera);

		mCamera->setPosition(CAMERA_POSITION);
        mCamera->lookAt(CAMERA_LOOK_AT);
        mCamera->setNearClipDistance(CAMERA_NEAR_CLIP_DISTANCE);

        mCameraNode->setPosition(CAMERA_NODE_POSITION);
	}

	virtual void createViewports()
	{
		Viewport* vp = mWindow->addViewport(mCamera);
		vp->setBackgroundColour(ColourValue(0,0,0));
		mCamera->setAspectRatio(Real(vp->getActualWidth()) / Real(vp->getActualHeight()));
	}

	virtual void createScene()
	{
		Entity* ent;
		Light* light;

		// Choix de la lumière et de la technique de shadowing
		mSceneMgr->setAmbientLight(AMBIENT_LIGHT);
		mSceneMgr->setShadowTechnique(SHADOWTYPE_STENCIL_ADDITIVE);

		// Chargement du terrain
		mSceneMgr->setWorldGeometry("terrain.cfg");

		// Chargement du skydome
		mSceneMgr->setSkyDome(true, "CloudySky");

		// Création de l'entity de l'avion
		ent = mSceneMgr->createEntity("Avion", "Avion.mesh");
		ent->setCastShadows(true);

		// Création du noeud de l'avion
		SceneNode* avionNode = mCameraNode->createChildSceneNode("AvionNode");
		avionNode->attachObject(ent);

		// Création de la light du sun
		light = mSceneMgr->createLight("Sun");
		light->setType(Light::LT_DIRECTIONAL);
		light->setDirection(LIGHT_DIRECTION);

		// Création du ParticleSystem de la neige
		ParticleSystem* neige = mSceneMgr->createParticleSystem("Neige", "NeigeParticle");

		// Création du SceneNode de la neige
		SceneNode* neigeNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("NeigeNode");
		neigeNode->attachObject(neige);
	}
};

#endif // DEMO_APPLICATION_H
